###############################################################################
## Please cite and acknowledge                                               ##
##  Lambert, P. and Bremhorst, V. (2018)                                     ##
##    Estimation and identification issues in the promotion time cure        ##
##    model when the same covariates influence long- and short-term survival ##
##  Biometrical Journal (2018)                                               ##
##                                                                           ##
## when using part of this code in your work and publications                ##
###############################################################################


######################################################################################
## GOAL: redo the colon dataset analysis with decreasing maximum follow-up times <xr>
##       to obtain Figure 2 in the paper
######################################################################################

###### Data reading ######
source("ColonCancer_DataReading.R")
obsTime.start = obsTime ; delta.start = delta  ## Rough data (before definition of our intermediate endpoint)

## Redo the analysis with max follow-up artificially set from year 1 to year 9 every 6 months
xr.grid = c(seq(1,9,by=.5), 9.120548)
nxr = length(xr.grid)

model.choice = 4 ## <----- SPECIFY covariates in the promotion time model (Model 4 corresponds to Table 5 in the paper)
ncoef = 9 ## <--- Nbr of regression coef in the selected model

summaryBayesEst.regr.array = summaryBayesEst.psidpsi.array = array(dim=c(ncoef,8,nxr))
for (kk in 1:nxr){
    xr = xr.grid[kk]  ## <----- Redefine the max follow-up duration
    source("ColonCancer_main.R") ## Estimate model parameters for these data
    summaryBayesEst.regr.array[,,kk] = as.matrix(summaryBayesEst.regr[,3:10])
    summaryBayesEst.psidpsi.array[,,kk] = as.matrix(summaryBayesEst.psidpsi[,3:10])
}
dimnames(summaryBayesEst.regr.array) = list(rownames(summaryBayesEst.regr),colnames(summaryBayesEst.regr)[3:10],xr.grid)
dimnames(summaryBayesEst.psidpsi.array) = list(rownames(summaryBayesEst.psidpsi),colnames(summaryBayesEst.psidpsi)[3:10],xr.grid)

## file.sav = "ColonCancer_VaryingXr.sav" ; save(summaryBayesEst.regr.array, summaryBayesEst.psidpsi.array, xr.grid, nxr, file=file.sav)
## file.Rdata = "ColonCancer_VaryingXr.Rdata" ; save.image(file=file.Rdata)


#### Corresponding Cox regression estimates
cox.hat.mat = cox.se.mat = matrix(nrow=6,ncol=nxr) ## 5 regression coef in the final model (Cox PH version)
for (kk in 1:nxr){
    xr = xr.grid[kk]  ## <-----
    cat("Maximum follow-up time <xr> set to",xr,"\n")
    ## Response <obsTime> and event indicator <delta>
    delta = ifelse(obsTime.start > xr, 0,delta.start) ## Change <delta> to 0 if obsTime > xr
    obsTime = pmin(obsTime.start,xr) ## Change <obsTime> to xr if  obsTime > xr
    ## Fit Cox PH
    Design.df = as.data.frame(Design)
    coxfit = with(Design.df, coxph(Surv(obsTime,delta) ~ TrtLev5FU +Nodes3+Nodes4 +DifferPoor +Extent12+Extent4))
    cox.hat.mat[,kk] = cox.hat = coxfit$coef
    cox.se.mat[,kk] = cox.se = sqrt(diag(coxfit$var))
}
rownames(cox.hat.mat) = rownames(cox.se.mat) = names(cox.hat)
colnames(cox.hat.mat) = colnames(cox.se.mat) = xr.grid


##### Plot of paired beta and gamma coefficients
ylims = range(summaryBayesEst.regr.array[-1,c(1,4,7),])  ## Range of regr coefs (except beta0)
ylims = c(-2,2)

## Fig 2.a
## ColonCancerBeta2Gamma2VaryingTr.pdf
par(mfrow=c(1,3),mar=c(4,4,2,1),cex=1.1)
xlab = expression(t[R])  ##expression(paste("Maximum follow-up duration ",t[R])),
##
lab = "beta2" ;  lab2=expression(beta[2]) ; main = "#Positive lymph nodes in [3,5]"
temp = summaryBayesEst.regr.array[c(lab),c(1,4,7),]
plot(xr.grid,temp["Mean",],yli=ylims,type="n",
     xlab=xlab,
     ylab=lab2)
axis(1,at=unique(round(xr.grid)))
title(main,line=-1,outer=TRUE)
polygon(c(xr.grid,rev(xr.grid)),c(temp["2.5%",],rev(temp["97.5%",])),col="grey",border=F,ylim=ylims)
abline(h=0,lty="dotted",lwd=2)
lines(xr.grid,temp["Mean",],lwd=2)
##
lab = "gamma2" ;  lab2=expression(gamma[2])
temp = summaryBayesEst.regr.array[c(lab),c(1,4,7),]
plot(xr.grid,temp["Mean",],yli=ylims,type="n",
     xlab=xlab,
     ylab=lab2)
axis(1,at=unique(round(xr.grid)))
polygon(c(xr.grid,rev(xr.grid)),c(temp["2.5%",],rev(temp["97.5%",])),col="grey",border=F,ylim=ylims)
abline(h=0,lty="dotted",lwd=2)
lines(xr.grid,temp["Mean",],lwd=2)
##
## lab = "psi2" ;  lab2=expression(psi[2])
## temp = summaryBayesEst.psidpsi.array[c(lab),c(1,4,7),]
lab = "psi2" ;  lab2=expression(beta[2]+gamma[2]) ## Take 2*psi_k = (beta_k + gamma_k)
temp = 2*summaryBayesEst.psidpsi.array[c(lab),c(1,4,7),]  ## Take 2*psi_k = (beta_k + gamma_k)
plot(xr.grid,temp["Mean",],yli=ylims,type="n",
     xlab=xlab,
     ylab=lab2)
axis(1,at=unique(round(xr.grid)))
polygon(c(xr.grid,rev(xr.grid)),c(temp["2.5%",],rev(temp["97.5%",])),col="grey",border=F,ylim=ylims)
abline(h=0,lty="dotted",lwd=2)
lines(xr.grid,temp["Mean",],lwd=2)
lines(xr.grid,cox.hat.mat["Nodes3",],lty="dashed",lwd=2) ## Superpose the Cox regression estimate

## Fig 2.b
## ColonCancerBeta3Gamma3VaryingTr.pdf
par(mfrow=c(1,3),mar=c(4,4,2,1),cex=1.1)
xlab = expression(t[R])
##
lab = "beta3" ;  lab2=expression(beta[3]) ; main = "#Positive lymph nodes >= 6"
temp = summaryBayesEst.regr.array[c(lab),c(1,4,7),]
plot(xr.grid,temp["Mean",],yli=ylims,type="n",
     xlab=xlab,
     ylab=lab2)
axis(1,at=unique(round(xr.grid)))
title(main,line=-1,outer=TRUE)
polygon(c(xr.grid,rev(xr.grid)),c(temp["2.5%",],rev(temp["97.5%",])),col="grey",border=F,ylim=ylims)
abline(h=0,lty="dotted",lwd=2)
lines(xr.grid,temp["Mean",],lwd=2)
##
lab = "gamma3" ;  lab2=expression(gamma[3])
temp = summaryBayesEst.regr.array[c(lab),c(1,4,7),]
plot(xr.grid,temp["Mean",],yli=ylims,type="n",
     xlab=xlab,
     ylab=lab2)
axis(1,at=unique(round(xr.grid)))
polygon(c(xr.grid,rev(xr.grid)),c(temp["2.5%",],rev(temp["97.5%",])),col="grey",border=F,ylim=ylims)
abline(h=0,lty="dotted",lwd=2)
lines(xr.grid,temp["Mean",],lwd=2)
##
## lab = "psi3" ;  lab2=expression(psi[3])
## temp = summaryBayesEst.psidpsi.array[c(lab),c(1,4,7),]
lab = "psi3" ;  lab2=expression(beta[3]+gamma[3]) ## Take 2*psi_k = (beta_k + gamma_k)
temp = 2*summaryBayesEst.psidpsi.array[c(lab),c(1,4,7),]  ## Take 2*psi_k = (beta_k + gamma_k)
plot(xr.grid,temp["Mean",],ylim=ylims,type="n",
     xlab=xlab,
     ylab=lab2)
axis(1,at=unique(round(xr.grid)))
polygon(c(xr.grid,rev(xr.grid)),c(temp["2.5%",],rev(temp["97.5%",])),col="grey",border=F,ylim=ylims)
abline(h=0,lty="dotted",lwd=2)
lines(xr.grid,temp["Mean",],lwd=2)
lines(xr.grid,cox.hat.mat["Nodes4",],lty="dashed",lwd=2) ## Superpose the Cox regression estimate


## Fig 2.c
##### Plot of "isolated" regression coefficients
## ColonCancerIsolatedRegrCoefVaryingTr.pdf
par(mfrow=c(1,4),mar=c(4,4,2,1),cex=1.1)
xlab = expression(t[R])
## beta1
lab = "beta1" ; lab2=expression(beta[1]) ; main = "Lev+5-FU"
temp = summaryBayesEst.regr.array[c(lab),c(1,4,7),]
plot(xr.grid,temp["Mean",],yli=ylims,type="n",
     xlab=xlab,
     ylab=lab2)
axis(1,at=unique(round(xr.grid)))
title(main)
polygon(c(xr.grid,rev(xr.grid)),c(temp["2.5%",],rev(temp["97.5%",])),col="grey",border=F,ylim=ylims)
abline(h=0,lty="dotted",lwd=2)
lines(xr.grid,temp["Mean",],lwd=2)
lines(xr.grid,cox.hat.mat["TrtLev5FU",],lty="dashed",lwd=2) ## Superpose the Cox regression estimate
## beta5
lab = "beta5" ; lab2=expression(beta[5]) ; main = "Submucosa / Muscle"
temp = summaryBayesEst.regr.array[c(lab),c(1,4,7),]
plot(xr.grid,temp["Mean",],yli=ylims,type="n",
     xlab=xlab,
     ylab=lab2)
axis(1,at=unique(round(xr.grid)))
title(main)
polygon(c(xr.grid,rev(xr.grid)),c(temp["2.5%",],rev(temp["97.5%",])),col="grey",border=F,ylim=ylims)
abline(h=0,lty="dotted",lwd=2)
lines(xr.grid,temp["Mean",],lwd=2)
lines(xr.grid,cox.hat.mat["Extent12",],lty="dashed",lwd=2) ## Superpose the Cox regression estimate
## beta6
lab = "beta6" ; lab2=expression(beta[6]) ; main = "Contiguous structures"
temp = summaryBayesEst.regr.array[c(lab),c(1,4,7),]
plot(xr.grid,temp["Mean",],yli=ylims,type="n",
     xlab=xlab,
     ylab=lab2)
axis(1,at=unique(round(xr.grid)))
title(main)
polygon(c(xr.grid,rev(xr.grid)),c(temp["2.5%",],rev(temp["97.5%",])),col="grey",border=F,ylim=ylims)
abline(h=0,lty="dotted",lwd=2)
lines(xr.grid,temp["Mean",],lwd=2)
lines(xr.grid,cox.hat.mat["Extent4",],lty="dashed",lwd=2) ## Superpose the Cox regression estimate
## gamma4
lab = "gamma4" ; lab2=expression(gamma[4]) ; main = "Poor cell diff."
temp = summaryBayesEst.regr.array[c(lab),c(1,4,7),]
plot(xr.grid,temp["Mean",],yli=ylims,type="n",
     xlab=xlab,
     ylab=lab2)
axis(1,at=unique(round(xr.grid)))
title(main)
polygon(c(xr.grid,rev(xr.grid)),c(temp["2.5%",],rev(temp["97.5%",])),col="grey",border=F,ylim=ylims)
abline(h=0,lty="dotted",lwd=2)
lines(xr.grid,temp["Mean",],lwd=2)
##lines(xr.grid,cox.hat.mat["DifferPoor",],lty="dashed",lwd=2) ## Superpose the Cox regression estimate

