###############################################################################
## Please cite and acknowledge                                               ##
##  Lambert, P. and Bremhorst, V. (2018)                                     ##
##    Estimation and identification issues in the promotion time cure        ##
##    model when the same covariates influence long- and short-term survival ##
##  Biometrical Journal (2018)                                               ##
##                                                                           ##
## when using part of this code in your work and publications                ##
###############################################################################

#########################################################################################
## Make your choice here:
## Define model choice for the analysis (see below for model definition)
model.choice = 0 ##  <---- Make your choice (Model 4 corresponds to Table 5 in the paper)
                 ##                         (Model 0 corresponds to Table 4 in the paper)
#########################################################################################


## Set minimum and maximum plausible values for reference cure probability here !
## Do that on log(-log(.)) scale
## Prior on P(cured|x=0) (such that in (p0.min,p0.max) with prob 0.95)
p0.min = .20 ; p0.max = .80  ##  <---- Make your choice

pcured.min = log(-log(p0.max)) ; pcured.max = log(-log(p0.min))

## Std deviation for priors of regression coefficients
s0.regr = 1.5

## Data reading
## ------------
source("ColonCancer_DataReading.R")
obsTime.start = obsTime ; delta.start = delta  ## Rough data (before definition of our intermediate endpoint)

## Support of the event time distribution
tmin = 0 ; tmax = max(obsTime)

## Interval (xl,xr) up to the maximum follow-up time <xr>
xl <- 0
if (!exists("xr")) xr = tmax
cat("Maximum follow-up time <xr> set to",xr,"\n")

## Response <obsTime> and event indicator <delta>
delta = ifelse(obsTime > xr, 0,delta) ## Change <delta> to 0 if obsTime > xr
obsTime = pmin(obsTime,xr) ## Change <obsTime> to xr if  obsTime > xr

## Design matrix
## -------------
Design.start = cbind(Intcpt = 1, TrtLev=TrtLev,TrtLev5FU=TrtLev5FU, Male=Male, Agec=Agec, Obstruct=Obstruct, Perfor=Perfor, Adhere=Adhere, Nodes2=Nodes2,Nodes3=Nodes3,Nodes4=Nodes4, DifferPoor=DifferPoor, Extent12=Extent12,Extent4=Extent4)
ndesign = ncol(Design.start)


##############################################################################
## Preliminary variable selection using Cox model (with xr = tmax = 9.120548):
##  R code and output
##############################################################################
my.data = data.frame(obsTime,delta,TrtLev,TrtLev5FU,Male,Agec,Obstruct,Perfor,Adhere,Nodes2,Nodes3,Nodes4,DifferPoor,Extent12,Extent4)
my.variable.list = c("TrtLev","TrtLev5FU",
                     "Male",
                     "Agec",
                     "Obstruct",
                     "Perfor",
                     "Adhere",
                     "Nodes2","Nodes3","Nodes4",
                     "DifferPoor",
                     "Extent12","Extent4")

## Code used to perform covariate selection using the Cox PH model
##################################################################
##
## library(My.stepwise)
## CoxModel.select = My.stepwise.coxph(Time="obsTime",Status="delta",variable.list=my.variable.list,data=my.data)
##
## *** Stepwise Final Model (in.lr.test: sle = 0.15; out.lr.test: sls = 0.15; variable selection restrict in vif = 999):
## Call:
## coxph(formula = Surv(obsTime, delta) ~ Nodes4 + TrtLev5FU + Nodes3 +
##     Extent12 + DifferPoor + Extent4 + Adhere + Obstruct, data = data,
##     method = "efron")
##
##   n= 888, number of events= 446
##
##               coef exp(coef) se(coef)      z Pr(>|z|)
## Nodes4      0.9522    2.5915   0.1171  8.134 4.44e-16 ***
## TrtLev5FU  -0.5093    0.6009   0.1092 -4.665 3.09e-06 ***
## Nodes3      0.5120    1.6686   0.1146  4.469 7.86e-06 ***
## Extent12   -0.5167    0.5965   0.1721 -3.003  0.00268 **
## DifferPoor  0.2862    1.3313   0.1236  2.315  0.02061 *
## Extent4     0.4072    1.5026   0.2137  1.905  0.05673 .
## Adhere      0.2073    1.2304   0.1278  1.622  0.10476
## Obstruct    0.1754    1.1918   0.1180  1.487  0.13696
## ---
## Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
##
##            exp(coef) exp(-coef) lower .95 upper .95
## Nodes4        2.5915     0.3859    2.0602    3.2598
## TrtLev5FU     0.6009     1.6641    0.4852    0.7443
## Nodes3        1.6686     0.5993    1.3330    2.0886
## Extent12      0.5965     1.6765    0.4257    0.8357
## DifferPoor    1.3313     0.7511    1.0449    1.6963
## Extent4       1.5026     0.6655    0.9884    2.2842
## Adhere        1.2304     0.8128    0.9578    1.5806
## Obstruct      1.1918     0.8391    0.9458    1.5018
##
## Concordance= 0.662  (se = 0.014 )
## Rsquare= 0.133   (max possible= 0.998 )
## Likelihood ratio test= 126.9  on 8 df,   p=0
## Wald test            = 125.1  on 8 df,   p=0
## Score (logrank) test = 132.7  on 8 df,   p=0
##
## --------------- Variance Inflating Factor (VIF) ---------------
## Multicollinearity Problem: Variance Inflating Factor (VIF) is bigger than 10 (Continuous Variable) or is bigger than 2.5 (Categorical Variable)
##     Nodes4  TrtLev5FU     Nodes3   Extent12 DifferPoor    Extent4     Adhere
##   1.126230   1.006179   1.118288   1.017291   1.021195   1.046114   1.047997
##   Obstruct
##   1.017155


## Covariates concerned by selection using Cox PH models (remove when Pcal > .10)
cov.select = c("TrtLev","TrtLev5FU",
               "Nodes2","Nodes3","Nodes4",
               "DifferPoor",
               "Extent12","Extent4")

Design.ref = Design.start[,c("Intcpt",cov.select)]

#########################################################
#####  LINEAR PREDICTORS  in cure promotion time model ##
#########################################################

cat("-------------\n")
cat("xr=",xr," ; Model ",model.choice,"\n",sep="")
cat("-------------\n")

#### P A R T  1 ###
## ----------------------------------------------------------
## Start with same covariate subsets in  Latency & Intensity
## ----------------------------------------------------------

##### Model 0: full model
#########################
## Latency = Intensity = TrtLev+TrtLev5FU +Nodes2+Nodes3+Nodes4 + DifferPoor + Extent12+Extent4
if (model.choice==0){
    col.beta  = c(1, 1,1, 1,1,1, 1, 1,1)  ## Intercept + selected covariates for latency part
    col.gamma = c(0, 1,1, 1,1,1, 1, 1,1)  ## No Intercept + Selected covariates for intensity part
}

## ## - xr = tmax = 9.12 -
## Selected covariates:
##            P(cured) Timing
## Intcpt            1      0
## TrtLev            1      1
## TrtLev5FU         1      1
## Nodes2            1      1
## Nodes3            1      1
## Nodes4            1      1
## DifferPoor        1      1
## Extent12          1      1
## Extent4           1      1

##               Var   Coef   Mean    SD    50%   2.5%     5%    95%  97.5%  Pval Sig EffSize
## psi0       Intcpt   psi0 -0.429 0.125 -0.429 -0.673 -0.632 -0.222 -0.184 0.000 ***     939
## psi1       TrtLev   psi1  0.006 0.076  0.008 -0.147 -0.118  0.129  0.155 0.458         785
## psi2    TrtLev5FU   psi2 -0.278 0.086 -0.277 -0.445 -0.419 -0.138 -0.112 0.000 ***    1237
## psi3       Nodes2   psi3  0.151 0.109  0.151 -0.064 -0.031  0.326  0.366 0.087   .     608
## psi4       Nodes3   psi4  0.440 0.096  0.439  0.255  0.286  0.597  0.625 0.000 ***     622
## psi5       Nodes4   psi5  0.656 0.097  0.655  0.465  0.497  0.815  0.848 0.000 ***     765
## psi6   DifferPoor   psi6  0.401 0.084  0.402  0.223  0.255  0.538  0.562 0.000 ***    1210
## psi7     Extent12   psi7 -0.297 0.124 -0.294 -0.555 -0.511 -0.095 -0.059 0.006  **    1465
## psi8      Extent4   psi8  0.400 0.142  0.403  0.118  0.165  0.629  0.670 0.004  **    1447
## dbeta1     TrtLev dbeta1 -0.069 0.115 -0.071 -0.294 -0.261  0.121  0.160 0.272        1168
## dbeta2  TrtLev5FU dbeta2 -0.243 0.127 -0.243 -0.490 -0.451 -0.033  0.010 0.030   *    1544
## dbeta3     Nodes2 dbeta3 -0.037 0.154 -0.038 -0.334 -0.285  0.218  0.273 0.398         793
## dbeta4     Nodes3 dbeta4  0.044 0.132  0.045 -0.214 -0.174  0.260  0.304 0.365         493
## dbeta5     Nodes4 dbeta5  0.219 0.138  0.220 -0.053 -0.009  0.448  0.489 0.055   .     467
## dbeta6 DifferPoor dbeta6 -0.302 0.118 -0.303 -0.530 -0.494 -0.105 -0.066 0.006  **    1640
## dbeta7   Extent12 dbeta7 -0.267 0.176 -0.277 -0.584 -0.536  0.037  0.121 0.068   .    1455
## dbeta8    Extent4 dbeta8 -0.125 0.211 -0.129 -0.527 -0.461  0.232  0.309 0.264        1627
##               Var   Coef   Mean    SD    50%   2.5%     5%    95%  97.5%  Pval Sig EffSize
## beta0      Intcpt  beta0 -0.429 0.125 -0.429 -0.673 -0.632 -0.222 -0.184 0.000 ***     939
## beta1      TrtLev  beta1 -0.064 0.118 -0.063 -0.299 -0.260  0.128  0.165 0.299        1480
## beta2   TrtLev5FU  beta2 -0.521 0.131 -0.519 -0.780 -0.738 -0.303 -0.264 0.000 ***    1668
## beta3      Nodes2  beta3  0.114 0.163  0.114 -0.209 -0.147  0.381  0.432 0.242        1184
## beta4      Nodes3  beta4  0.484 0.142  0.484  0.205  0.248  0.721  0.762 0.000 ***     993
## beta5      Nodes4  beta5  0.875 0.142  0.876  0.597  0.643  1.111  1.155 0.000 ***    1289
## beta6  DifferPoor  beta6  0.099 0.131  0.100 -0.165 -0.117  0.316  0.359 0.221        1597
## beta7    Extent12  beta7 -0.563 0.185 -0.558 -0.929 -0.870 -0.266 -0.201 0.002  **    1617
## beta8     Extent4  beta8  0.275 0.232  0.276 -0.184 -0.117  0.651  0.719 0.121        1611
## gamma1     TrtLev gamma1  0.075 0.156  0.076 -0.234 -0.181  0.328  0.378 0.318         785
## gamma2  TrtLev5FU gamma2 -0.034 0.173 -0.033 -0.379 -0.321  0.243  0.293 0.423        1318
## gamma3     Nodes2 gamma3  0.188 0.212  0.186 -0.230 -0.156  0.539  0.598 0.189         454
## gamma4     Nodes3 gamma4  0.396 0.181  0.394  0.034  0.094  0.699  0.755 0.016   *     380
## gamma5     Nodes4 gamma5  0.437 0.192  0.435  0.065  0.131  0.756  0.824 0.011   *     575
## gamma6 DifferPoor gamma6  0.703 0.158  0.703  0.388  0.443  0.956  1.005 0.000 ***    1331
## gamma7   Extent12 gamma7 -0.030 0.242 -0.014 -0.572 -0.455  0.334  0.398 0.475        1379
## gamma8    Extent4 gamma8  0.525 0.275  0.531 -0.044  0.056  0.960  1.033 0.034   *    1619
##
## DIC=2319.63 ; pD=21.49 ; D.avg=2298.15 ; D.hat=2276.66 ; BIC=2407.74

##### Model 1: discard some non significant contributions (already suggested by Cox PH model selection)
#########################################################
## Latency = Intensity =
if (model.choice==1){
    ## Intensity =    Intercept +  /+TrtLev5FU  +/+Nodes3+Nodes4 + DifferPoor + Extent12+Extent4
    col.beta  = c(1, 0,1, 0,1,1, 1, 1,1)
    ## Latency   = No Intercept +  /+TrtLev5FU  +/+Nodes3+Nodes4 + DifferPoor + Extent12+Extent4
    col.gamma = c(0, 0,1, 0,1,1, 1, 1,1)
}

## > print(t(mat))
##            P(cured) Timing
## Intcpt            1      0
## TrtLev            0      0
## TrtLev5FU         1      1
## Nodes2            0      0
## Nodes3            1      1
## Nodes4            1      1
## DifferPoor        1      1
## Extent12          1      1
## Extent4           1      1


##               Var   Coef   Mean    SD    50%   2.5%     5%    95%  97.5%  Pval Sig EffSize
## psi0       Intcpt   psi0 -0.418 0.090 -0.419 -0.604 -0.566 -0.269 -0.243 0.000 ***    1006
## psi1    TrtLev5FU   psi1 -0.280 0.079 -0.279 -0.438 -0.414 -0.152 -0.130 0.000 ***    1458
## psi2       Nodes3   psi2  0.363 0.079  0.362  0.205  0.233  0.489  0.518 0.000 ***     828
## psi3       Nodes4   psi3  0.584 0.081  0.582  0.427  0.451  0.720  0.745 0.000 ***    1126
## psi4   DifferPoor   psi4  0.404 0.084  0.402  0.237  0.268  0.542  0.567 0.000 ***    1416
## psi5     Extent12   psi5 -0.293 0.128 -0.291 -0.554 -0.506 -0.086 -0.046 0.010  **    1373
## psi6      Extent4   psi6  0.391 0.144  0.399  0.103  0.147  0.619  0.663 0.005  **    1522
## dbeta1  TrtLev5FU dbeta1 -0.216 0.107 -0.217 -0.417 -0.387 -0.036 -0.004 0.022   *    1421
## dbeta2     Nodes3 dbeta2  0.077 0.109  0.077 -0.133 -0.101  0.258  0.292 0.238        1059
## dbeta3     Nodes4 dbeta3  0.250 0.119  0.250  0.015  0.055  0.448  0.485 0.018   *    1113
## dbeta4 DifferPoor dbeta4 -0.302 0.113 -0.303 -0.519 -0.485 -0.114 -0.076 0.005  **    1576
## dbeta5   Extent12 dbeta5 -0.268 0.172 -0.276 -0.581 -0.534  0.021  0.097 0.062   .    1453
## dbeta6    Extent4 dbeta6 -0.112 0.211 -0.118 -0.508 -0.448  0.232  0.302 0.290        1673
##               Var   Coef   Mean    SD    50%   2.5%     5%    95%  97.5%  Pval Sig EffSize
## beta0      Intcpt  beta0 -0.418 0.090 -0.419 -0.604 -0.566 -0.269 -0.243 0.000 ***    1006
## beta1   TrtLev5FU  beta1 -0.495 0.117 -0.494 -0.725 -0.687 -0.304 -0.271 0.000 ***    1571
## beta2      Nodes3  beta2  0.441 0.118  0.442  0.206  0.248  0.637  0.672 0.000 ***    1448
## beta3      Nodes4  beta3  0.834 0.126  0.835  0.583  0.626  1.037  1.079 0.000 ***    1404
## beta4  DifferPoor  beta4  0.102 0.130  0.104 -0.156 -0.116  0.314  0.357 0.217        1535
## beta5    Extent12  beta5 -0.561 0.184 -0.563 -0.917 -0.857 -0.264 -0.198 0.002  **    1588
## beta6     Extent4  beta6  0.279 0.230  0.283 -0.186 -0.113  0.642  0.708 0.121        1602
## gamma1  TrtLev5FU gamma1 -0.064 0.147 -0.060 -0.357 -0.310  0.173  0.216 0.338        1389
## gamma2     Nodes3 gamma2  0.286 0.149  0.286 -0.010  0.040  0.523  0.569 0.028   *     846
## gamma3     Nodes4 gamma3  0.334 0.161  0.338  0.015  0.070  0.595  0.647 0.021   *     841
## gamma4 DifferPoor gamma4  0.706 0.151  0.709  0.404  0.455  0.950  1.000 0.000 ***    1445
## gamma5   Extent12 gamma5 -0.025 0.240 -0.013 -0.535 -0.428  0.344  0.408 0.478        1380
## gamma6    Extent4 gamma6  0.503 0.279  0.514 -0.061  0.031  0.944  1.027 0.040   *    1603
##
## DIC=2313.65 ; pD=17.40 ; D.avg=2296.24 ; D.hat=2278.84 ; BIC=2385.00


##### Model 2: specialize some covariate effects when clear (TrtLev5FU, DifferPoor)
###########################################################
## Latency = Intensity =
if (model.choice==2){
    ## Intensity =    Intercept +  /+TrtLev5FU  +/+Nodes3+Nodes4 +      /     + Extent12+Extent4
    col.beta  = c(1, 0,1, 0,1,1, 0, 1,1)
    ## Latency   = No Intercept +  /+/          +/+Nodes3+Nodes4 + DifferPoor + Extent12+Extent4
    col.gamma = c(0, 0,0, 0,1,1, 1, 1,1)
}

## > print(t(mat))
##            P(cured) Timing
## Intcpt            1      0
## TrtLev            0      0
## TrtLev5FU         1      0
## Nodes2            0      0
## Nodes3            1      1
## Nodes4            1      1
## DifferPoor        0      1
## Extent12          1      1
## Extent4           1      1

##               Var   Coef   Mean    SD    50%   2.5%     5%    95%  97.5%  Pval Sig EffSize
## psi0       Intcpt   psi0 -0.398 0.088 -0.398 -0.570 -0.542 -0.251 -0.221 0.000 ***    1087
## psi1    TrtLev5FU   psi1 -0.507 0.108 -0.506 -0.726 -0.687 -0.327 -0.295 0.000 ***    1586
## psi2       Nodes3   psi2  0.364 0.079  0.364  0.209  0.233  0.492  0.519 0.000 ***     777
## psi3       Nodes4   psi3  0.589 0.083  0.588  0.422  0.453  0.727  0.753 0.000 ***     927
## psi4   DifferPoor   psi4  0.734 0.144  0.733  0.448  0.492  0.967  1.011 0.000 ***    1290
## psi5     Extent12   psi5 -0.291 0.126 -0.290 -0.542 -0.493 -0.086 -0.048 0.008  **    1473
## psi6      Extent4   psi6  0.394 0.140  0.396  0.100  0.154  0.626  0.660 0.004  **    1471
## dbeta2     Nodes3 dbeta2  0.078 0.112  0.078 -0.140 -0.099  0.266  0.305 0.242         968
## dbeta3     Nodes4 dbeta3  0.259 0.123  0.258  0.020  0.060  0.465  0.504 0.018   *    1067
## dbeta5   Extent12 dbeta5 -0.273 0.177 -0.282 -0.593 -0.550  0.027  0.102 0.065   .    1548
## dbeta6    Extent4 dbeta6 -0.099 0.220 -0.109 -0.503 -0.448  0.267  0.357 0.312        1773
##               Var   Coef   Mean    SD    50%   2.5%     5%    95%  97.5%  Pval Sig EffSize
## beta0      Intcpt  beta0 -0.398 0.088 -0.398 -0.570 -0.542 -0.251 -0.221 0.000 ***    1087
## beta1   TrtLev5FU  beta1 -0.507 0.108 -0.506 -0.726 -0.687 -0.327 -0.295 0.000 ***    1586
## beta2      Nodes3  beta2  0.442 0.119  0.441  0.209  0.249  0.642  0.677 0.000 ***    1407
## beta3      Nodes4  beta3  0.848 0.127  0.847  0.599  0.642  1.062  1.099 0.000 ***    1457
## beta5    Extent12  beta5 -0.564 0.183 -0.560 -0.925 -0.867 -0.271 -0.211 0.001 ***    1862
## beta6     Extent4  beta6  0.295 0.236  0.303 -0.182 -0.101  0.662  0.729 0.106        1803
## gamma2     Nodes3 gamma2  0.286 0.153  0.285 -0.014  0.035  0.538  0.582 0.031   *     710
## gamma3     Nodes4 gamma3  0.329 0.166  0.332 -0.003  0.051  0.601  0.653 0.026   *     843
## gamma4 DifferPoor gamma4  0.734 0.144  0.733  0.448  0.492  0.967  1.011 0.000 ***    1290
## gamma5   Extent12 gamma5 -0.018 0.246 -0.009 -0.535 -0.441  0.357  0.417 0.488        1443
## gamma6    Extent4 gamma6  0.493 0.283  0.505 -0.112  0.009  0.932  1.015 0.047   *    1667

## DIC=2310.77 ; pD=15.43 ; D.avg=2295.35 ; D.hat=2279.92 ; BIC=2374.04


##### Model 3: regroup Extent4 and the reference category (Extent3) --> No Extent4
###################################################################
## Latency = Intensity =
if (model.choice==3){
    ## Intensity =    Intercept +  /+TrtLev5FU  +/+Nodes3+Nodes4 +      /     + Extent12+/
    col.beta  = c(1, 0,1, 0,1,1, 0, 1,0)
    ## Latency   = No Intercept +  /+/          +/+Nodes3+Nodes4 + DifferPoor + Extent12+/
    col.gamma = c(0, 0,0, 0,1,1, 1, 1,0)
}

## > print(t(mat))
##            P(cured) Timing
## Intcpt            1      0
## TrtLev            0      0
## TrtLev5FU         1      0
## Nodes2            0      0
## Nodes3            1      1
## Nodes4            1      1
## DifferPoor        0      1
## Extent12          1      1
## Extent4           0      0

##               Var   Coef   Mean    SD    50%   2.5%     5%    95%  97.5%  Pval Sig EffSize
## psi0       Intcpt   psi0 -0.375 0.084 -0.373 -0.540 -0.516 -0.238 -0.212 0.000 ***    1209
## psi1    TrtLev5FU   psi1 -0.510 0.109 -0.507 -0.732 -0.691 -0.337 -0.302 0.000 ***    1657
## psi2       Nodes3   psi2  0.368 0.078  0.369  0.210  0.240  0.494  0.519 0.000 ***     854
## psi3       Nodes4   psi3  0.589 0.082  0.588  0.430  0.455  0.724  0.748 0.000 ***    1038
## psi4   DifferPoor   psi4  0.739 0.145  0.742  0.446  0.494  0.971  1.013 0.000 ***    1259
## psi5     Extent12   psi5 -0.315 0.126 -0.308 -0.571 -0.530 -0.116 -0.074 0.005  **    1428
## dbeta2     Nodes3 dbeta2  0.069 0.114  0.068 -0.149 -0.113  0.258  0.295 0.275         910
## dbeta3     Nodes4 dbeta3  0.254 0.122  0.252  0.014  0.056  0.457  0.494 0.020   *    1069
## dbeta5   Extent12 dbeta5 -0.262 0.176 -0.275 -0.576 -0.527  0.049  0.117 0.073   .    1344
##               Var   Coef   Mean    SD    50%   2.5%     5%    95%  97.5%  Pval Sig EffSize
## beta0      Intcpt  beta0 -0.375 0.084 -0.373 -0.540 -0.516 -0.238 -0.212 0.000 ***    1209
## beta1   TrtLev5FU  beta1 -0.510 0.109 -0.507 -0.732 -0.691 -0.337 -0.302 0.000 ***    1657
## beta2      Nodes3  beta2  0.437 0.119  0.439  0.193  0.238  0.628  0.669 0.000 ***    1339
## beta3      Nodes4  beta3  0.843 0.126  0.842  0.598  0.635  1.050  1.095 0.000 ***    1542
## beta5    Extent12  beta5 -0.577 0.182 -0.574 -0.935 -0.877 -0.280 -0.228 0.001 ***    1696
## gamma2     Nodes3 gamma2  0.299 0.155  0.304 -0.011  0.043  0.547  0.589 0.028   *     741
## gamma3     Nodes4 gamma3  0.335 0.166  0.336  0.008  0.066  0.603  0.652 0.023   *     921
## gamma4 DifferPoor gamma4  0.739 0.145  0.742  0.446  0.494  0.971  1.013 0.000 ***    1259
## gamma5   Extent12 gamma5 -0.053 0.246 -0.031 -0.599 -0.496  0.310  0.382 0.445        1407

## DIC=2314.32 ; pD=13.20 ; D.avg=2301.12 ; D.hat=2287.92 ; BIC=2368.43


#################################
##### F I N A L   M O D E L #####
#################################

##### Model 4: come back with Extent4 AND specialize the effect of Extent4 and Extent12
#######################################################################################
## Latency = Intensity =
if (model.choice==4){
    ## Intensity =    Intercept +  /+TrtLev5FU  +/+Nodes3+Nodes4 +      /     + Extent12+Extent4
    col.beta  = c(1, 0,1, 0,1,1, 0, 1,1)
    ## Latency   = No Intercept +  /+/          +/+Nodes3+Nodes4 + DifferPoor +        /+/
    col.gamma = c(0, 0,0, 0,1,1, 1, 0,0)
}

## > print(t(mat))
##            P(cured) Timing
## Intcpt            1      0
## TrtLev            0      0
## TrtLev5FU         1      0
## Nodes2            0      0
## Nodes3            1      1
## Nodes4            1      1
## DifferPoor        0      1
## Extent12          1      0
## Extent4           1      0

##               Var   Coef   Mean    SD    50%   2.5%     5%    95%  97.5%  Pval Sig EffSize
## psi0       Intcpt   psi0 -0.403 0.085 -0.402 -0.574 -0.548 -0.266 -0.245 0.000 ***    1192
## psi1    TrtLev5FU   psi1 -0.506 0.108 -0.504 -0.719 -0.687 -0.335 -0.301 0.000 ***    1937
## psi2       Nodes3   psi2  0.375 0.077  0.376  0.218  0.248  0.503  0.527 0.000 ***     887
## psi3       Nodes4   psi3  0.597 0.079  0.596  0.443  0.466  0.727  0.753 0.000 ***    1070
## psi4   DifferPoor   psi4  0.733 0.147  0.735  0.438  0.491  0.971  1.020 0.000 ***    1260
## psi5     Extent12   psi5 -0.572 0.171 -0.564 -0.912 -0.858 -0.289 -0.241 0.000 ***    1699
## psi6      Extent4   psi6  0.462 0.205  0.471  0.038  0.112  0.785  0.837 0.017   *    1601
## dbeta2     Nodes3 dbeta2  0.071 0.110  0.072 -0.148 -0.112  0.248  0.284 0.256         936
## dbeta3     Nodes4 dbeta3  0.250 0.116  0.250  0.024  0.062  0.446  0.481 0.016   *     747
##               Var   Coef   Mean    SD    50%   2.5%     5%    95%  97.5%  Pval Sig EffSize
## beta0      Intcpt  beta0 -0.403 0.085 -0.402 -0.574 -0.548 -0.266 -0.245 0.000 ***    1192
## beta1   TrtLev5FU  beta1 -0.506 0.108 -0.504 -0.719 -0.687 -0.335 -0.301 0.000 ***    1937
## beta2      Nodes3  beta2  0.446 0.117  0.447  0.214  0.252  0.635  0.669 0.000 ***    1514
## beta3      Nodes4  beta3  0.846 0.124  0.848  0.604  0.641  1.052  1.089 0.000 ***    1541
## beta5    Extent12  beta5 -0.572 0.171 -0.564 -0.912 -0.858 -0.289 -0.241 0.000 ***    1699
## beta6     Extent4  beta6  0.462 0.205  0.471  0.038  0.112  0.785  0.837 0.017   *    1601
## gamma2     Nodes3 gamma2  0.304 0.149  0.306  0.000  0.051  0.544  0.591 0.025   *     681
## gamma3     Nodes4 gamma3  0.347 0.156  0.350  0.040  0.091  0.605  0.649 0.012   *     808
## gamma4 DifferPoor gamma4  0.733 0.147  0.735  0.438  0.491  0.971  1.020 0.000 ***    1260

## DIC=2308.98 ; pD=12.97 ; D.avg=2296.01 ; D.hat=2283.03 ; BIC=2362.17

#################################
##########  Model fit  ##########
#################################
cat("==============\n")
cat("  Model No ",model.choice,"\n",sep="")
cat("==============\n")

## Remind selected covariates
mat = rbind(col.beta,col.gamma)
colnames(mat) = colnames(Design.ref)
rownames(mat) = c("P(cured)","Timing")
cat("Selected covariates:\n")
print(t(mat))

## Prepare Design matrix with selected covariates
col.select = which(col.beta+col.gamma>=1)
Design = Design.ref[,col.select]  ## Define design matrix based on activated covariates
ndesign = ncol(Design)
X.col = col.beta[col.select] ; W.col = col.gamma[col.select]

## Default parameter estimation procedure (also used during simulation)
set.seed(1234)
source("FlexibleCureModel_main.R")

## DIC
D.hat = -2*attr(logpost.regr(theta.MAP,obsTime,delta),"llik")
D.avg = mean(-2*llik.vec[-burnin])
p.D = D.avg-D.hat
DIC = D.hat + 2*p.D
BIC = D.hat + log(sum(delta==1))*p.D ## Use  BIC = deviance  + log(#events)*pD (instead of log(n)*pD) (Volinsky & Raftery, Biometrics 2000)
message(sprintf("DIC=%.2f ; pD=%.2f ; D.avg=%.2f ; D.hat=%.2f ; BIC=%.2f",DIC,p.D,D.avg,D.hat,BIC))

## Save workspave
## file.Rdata = paste("ConlonCancer_xr",xr,"_Model",model.choice,".Rdata",sep="") ; save.image(file=file.Rdata)

cat("\n---------------------------------------------------------------------------\n")
cat("Please cite\n")
cat("  Lambert, P. and Bremhorst, V. (2018)\n")
cat("    Estimation and identification issues in the promotion time cure\n")
cat("    model when the same covariates influence long- and short-term survival\n")
cat("  Biometrical Journal (2018)\n")
cat("----------------------------------------------------------------------------\n")
