###############################################################################
## Please cite and acknowledge                                               ##
##  Lambert, P. and Bremhorst, V. (2018)                                     ##
##    Estimation and identification issues in the promotion time cure        ##
##    model when the same covariates influence long- and short-term survival ##
##  Biometrical Journal (2018)                                               ##
##                                                                           ##
## when using part of this code in your work and publications                ##
###############################################################################

#########################################################################################################
##  CAREFUL CAREFUL CAREFUL !!!
## FlexibleCureModel_BayesianEstimation.R is designed to be sourced from FlexibleCureModel_main.R !!
#########################################################################################################

## set.seed(1234)
library(coda)
bayes.estimation = TRUE

## Automatic tuning of sigma during the burnin of the Metropolis algorithm
# Goal : indicates whether one accepts the proposed M-H step
#        and updates the std deviation of the normal shocks if
#        the iteration number m > 5 and m < burnin.
#
# Input:
#        pcur: current value of p of length np
#        lcur: current value of logpost
#        pprop: proposed value for p of length np
#        lprop: proposed value for logpost
#        m : iteration number
#        burnin : length of burnin
#        sigma0 : std deviation of normal shocks at iteration 1
#        sigmacur : std deviation of normal shocks at current iteration
#        targetRate : targetted acceptance rate in M-H algorithm
#
# Output:
#        pcur : becomes pprop if acceptance, unchanged otherwise
#        lcur : becomes lprop if acceptance, unchanged otherwise
#        sigmacur: new std deviation after this M-H step
#        accept : 0-1 indicator of acceptance of M-H step
#
# -----------------------------------
# Philippe LAMBERT (ULiege, Dec 2014)
# Email:  p.lambert@uliege.be
# Web: http://www.statsoc.ulg.ac.be
# -----------------------------------
MHstep = function(pcur, lcur, pprop, lprop, m, burnin, sigma0, sigma.cur, targetRate){
  lalpha = lprop - lcur
  if (is.nan(lalpha)) lalpha=-100
  if (lalpha > 0)  alpha = 1.
  else alpha = exp(lalpha)
  #
  if (runif(1) <= alpha){
    pcur = pprop
    lcur = lprop
    accept = 1
  } else  accept = 0
  #
  if ((m > 5) & (m < burnin)){
     sigma.cur = sigma.cur + sigma0 * (alpha - targetRate)/sqrt(m)
     if (sigma.cur <= 0.) sigma.cur = .000001
   }
  return(list(pcur=pcur, lcur=lcur, sigma.cur=sigma.cur, accept=accept))
}

## Goal: compute the (Log-)posterior function for the flexible promotion time cure model
##        (cf. Section 2.5 in the paper)
##
## Input:
##        alpha: vector of spline, regression and penalty parameters
##        obsTime: vector of event or censoring times
##        delta: vector of event indicators
##
## Output:
##        lpost: log-posterior (with log-likelihood in attribute)
##
##
## Note:  spline parameters <phi> are in the <zeta> parametrization:
##          phi = zeta2phi(zeta)
##
## -----------------------------------
## Philippe LAMBERT (ULiege, May 2017)
## Email:  p.lambert@uliege.be
## Web: http://www.statsoc.ulg.ac.be
## -----------------------------------
##
logpost.regr = function(alpha,obsTime,delta,phi.max=2,noprior=FALSE){
    nu = 1 ; aa.xi = bb.xi = .5 ## Hyperprior parms for penalty parms
    ## Parameters
    zeta = alpha[zeta.lab] ; phi = zeta2phi(zeta,phi.max=phi.max) ## Spline parameters
    if (any(!is.finite(phi))) return(-1e10)
    KK = length(phi)
    ##
    psi = alpha[psi.lab]
    ##
    ## Penalty parameters (Jullion & Lambert, 2007, after marginalization with nu=1, aa.xi=bb.xi=.5)
    lambda.zeta = alpha["penalty1"]  # lambda \sim G(.5*nu,.5*nu*xi)
    xi = alpha["penalty2"]           # xi \sim G(aa.xi,bb.xi)
    ##
    dbeta = 0*psi
    ndbeta = sum(X.col*W.col)
    if (ndbeta > 0) dbeta[(X.col*W.col)==1] = alpha[dbeta.lab]
    ##
    beta = psi*X.col + dbeta
    gamma = psi*W.col - dbeta
    ##
    eta1 = c(Design%*%beta) ## log(theta(x)) = linear predictor in long-term survival model (cf. Eq.(1) in paper)
    eta2 = c(Design%*%gamma)## z'gamma = linear predictor in short-term survival model (cf. Eq.(2) in paper)
    temp = h0H0.fun(obsTime,phi)
    lh0.t = log(temp$h0) ; H0.t = temp$H0 ## Baseline (log-)hazard and cumulative hazard
    lhp.t = eta1 + eta2 + lh0.t - exp(eta2)*H0.t ## Population (log-)hazard
    lSp.t = -exp(eta1)*(1-exp(-exp(eta2)*H0.t))  ## Population (log-)survival
    ##
    llik = sum(delta*lhp.t + lSp.t) ## (log-)likelihood
    ##
    ## Roughness penalty
    quad.zeta = sum(zeta*c(Pd %*% zeta))
    lpen = .5*KK*log(lambda.zeta) -.5*lambda.zeta*quad.zeta ## p(zeta|tau)
    lpen = lpen + dgamma(lambda.zeta,.5*nu,.5*nu*xi,log=T)  ## p(lambda|xi), cf. Penalty parameter
    lpen = lpen + dgamma(xi,aa.xi,bb.xi,log=T)              ## p(xi)
    ##
    ## Encourage S0(tmax) in (1e-6, 1e-2)
    Stmax = exp(-h0H0.fun(tmax,phi)$H0)
    lpen2 = 0
    lpen2 = log(n)*dnorm(log10(Stmax),-4,1,log=T)
    ##
    ## Prior on 'active' component in dbeta's
    lprior = 0
    if (ndbeta > 0){
        parms = dbeta[(X.col*W.col)==1]
        lpen.dbeta = sum(dnorm(parms,0*parms,s0.regr,log=T)) ##  <----------------- (prior sd=s0.regr)
        lprior = lprior + lpen.dbeta
    }
    ## Prior on <psi>
    lprior = lprior + dnorm(psi[1],b0.mean,b0.sd,log=T)   ## psi[1] = beta0 = log(-log(P(cured|x=0)))
    lprior = lprior + sum(dnorm(psi[-1],0,s0.regr,log=T)) ## Prior on <psi[-1]>
    ##
    if (noprior) lprior = 0
    lpost = llik + lpen + lpen2 + lprior ## (log-)posterior
    names(lpost) = "lpost.regr"
    attr(lpost,"llik") = llik
    return(lpost)
}

## Function for Gibbs step for the penalty parameters <lambda.zeta> and <xi>
rgibbs = function(theta.label,theta,fixed=NULL){
    nu = 1 ; aa.xi = bb.xi = .5
    #
    ans = NULL
    # Penalty parameters
    if (theta.label=="penalty"){
        zeta = theta[zeta.lab]
        xi = theta["penalty2"]
        quad.zeta = sum(zeta*c(Pd %*%zeta))
        lambda.zeta = rgamma(1,.5*(nu+KK),.5*(nu*xi+quad.zeta))
        xi = rgamma(1, aa.xi+.5*nu, bb.xi+.5*nu*lambda.zeta)
        ans = c(penalty1=as.numeric(lambda.zeta),penalty2=as.numeric(xi))
    }
    #
    return(ans)
}


## Compute initial values for the spline and regression parameters
## ---------------------------------------------------------------
## Idea: minimize minus the log-posterior for fixed penalty parameters
fun = function(coef){
    theta = c(coef,10,1)
    names(theta) = c(zeta.lab,psi.lab,dbeta.lab,"penalty1","penalty2")
    ans = -logpost.regr(theta,obsTime,delta)
    return(ans)
}
##
penalty0 = c(penalty1=10,penalty2=1) ## Fixed penalty parameters
coef00 = c(alpha0[zeta.lab],alpha0[psi.lab],alpha0[dbeta.lab])
coef.lab = names(coef00)
##
obj0 = nlm(fun,coef00,typsize=coef00,hessian=T,iterlim=200) ## Optimization
coef0 = obj0$est ; ncoef = length(coef0) ## Obtained initial values
names(coef0) = names(coef00)

## Variance-covariance at the optimum
Sigma.coef0 = diag(0,length(coef0))
Sigma.coef0 = solve(obj0$hes) ##+1e-4*diag(ncol(obj0$hes)))

## Vector of inital values
theta0 = c(coef0,penalty0) ; ntheta = length(theta0)


## Metropolis-within-Gibbs: prepare for it
## ---------------------------------------
niter = 1.2e4 ## Number of iterations in MCMC
burnin = 2e3  ## Length of burn-in
##
targetRate = .35 ## Target acceptance rate in MCMC
#
use.chol = TRUE ## if FALSE, just use frequentist s.e.to make proposal in MCMC ;
                ## if TRUE, inverse the frequentist hessian and take cholesky in MCMC procedure

## Cholesky transform of Variance-covariance to make efficient proposals in MCMC
idx1 = 1:(KK) ## Spline parameters in first KK positions in <theta0>
idx2 = (KK+1):ncoef ## Position of the other parameters in <theta0>
if (!exists("s")) s=1
if (s==1){
    if (use.chol){
        Lcoef = 0*Sigma.coef0
        Lcoef[idx1,idx1] = svd(Pd)$u
        Lcoef[idx2,idx2] = t(chol(Sigma.coef0[idx2,idx2]))
    }
    else Lcoef = diag(sqrt(diag(Sigma.coef0)))
}

## Initial values for the model parameters
ntheta = length(theta0)
sigma0 = sigma.cur = rep(1,ncoef)

## Prepare space to save results during MCMC
lpost.vec = llik.vec = numeric(niter) ## ... for log-posterior
theta.chn = matrix(nrow=length(theta0),ncol=niter) ## ... for spline and regression parameters
penalty.chn = matrix(nrow=2,ncol=niter) ## for penalty parameters
rownames(theta.chn) = names(theta0) ; colnames(theta.chn) = 1:niter ## Labels for them
##
## Monitoring of quantiles of baseline survival function
if (exists("quant0")){
    S0quant0.chn = beta0b.chn = matrix(nrow=length(quant0),ncol=niter)
    rownames(S0quant0.chn) = names(quant0) ; colnames(S0quant0.chn) = 1:niter
}
##
## Monitoring of  beta0 + log(F0(t_R))
if (exists("beta0b.true")){
    rownames(beta0b.chn) = names(beta0b.true) ; colnames(beta0b.chn) = 1:niter
}
##
## Acceptance rates over iterations and tuning of proposals during burn-in
temp = matrix(nrow=ncoef,ncol=niter)
rownames(temp) = coef.lab ; colnames(temp) = 1:niter
accept.coef = sigma.chn = temp
##

## Metropolis-within-Gibbs: algorithm per se
## -----------------------------------------
m.Lcoef = 0
##
ptm <- proc.time()
##
theta.cur = theta0 ## Current value of the model parameters (here at the start)
lcur = logpost.regr(theta.cur,obsTime,delta) ## Value of the log-posterior at the start
##
for (m in 1:niter){
    if (m%%500==0) cat (m," ") ## Keep track of iteration number
    ##
    theta.prop = theta.cur
    ## Loop over the <ncoef> spline and regression parameter and make proposal for each in turn
    ##  (Note: it is made along the principal axes of the variance-covariance matrix)
    for (k in 1:ncoef){
        ## Compute the proposal
        theta.prop[coef.lab] = theta.cur[coef.lab] + sigma.cur[k]*Lcoef[,k]*rnorm(1)
        ## Value of log-posterior at proposal
        lprop = logpost.regr(theta.prop,obsTime,delta)
        ## Metropolis rule to accept or reject proposal
        ans = MHstep(theta.cur,lcur,theta.prop,lprop,m-m.Lcoef,burnin,sigma0[k],sigma.cur[k],targetRate)
        ## Update according to the decision during the Metropolis step
        theta.cur = ans$pcur ; lcur = ans$lcur ; sigma.cur[k] = ans$sigma.cur ; accept = ans$accept
        ## Tune standard deviation of proposals to achieve the targetted acceptance rate
        sigma.chn[k,m] = sigma.cur[k] ; accept.coef[k,m] = accept
    }
    ## Gibbs steps for the penalty parameters
    penalty.cur = rgibbs("penalty",theta.cur)
    theta.cur["penalty1"] = penalty.cur["penalty1"] ; theta.cur["penalty2"] = penalty.cur["penalty2"]
    ## Value of the log-posterior at the end of iteration <m>
    lcur = logpost.regr(theta.cur,obsTime,delta)
    ## Save the value of the model parameters at the end of iteration <m>
    theta.chn[,m] = theta.cur
    penalty.chn[,m] = penalty.cur
    lpost.vec[m] = lcur
    llik.vec[m] = attr(lcur,"llik")
    ## Translate chain in <zeta> in the <phi> parametrization (cf. spline parameters)
    phi.max = 2 ; phi.cur = zeta2phi(theta.cur[zeta.lab],phi.max=phi.max) ## phi.max - exp(theta.cur[zeta.lab])
    ## Monitoring of  beta0 + log(F0(t_R))
    if (exists("quant0")){
        S0quant0.chn[,m] =  exp(-h0H0.fun(quant0,phi.cur)$H0)
        beta0b.chn[,m] = theta.chn["psi0",m] + log(1-S0quant0.chn[,m])
    }
}
## Induced chain in the (beta,gamma) parametrization
regr.chn = psiToRegr %*% theta.chn[c(psi.lab,dbeta.lab),]
##
elapsed.time <- (proc.time()-ptm)[1]
cat("\nElapsed time: ",round(elapsed.time,1),"seconds\n\n")


## Extract information from MCMC chain (post burn-in)
## --------------------------------------------------
id.burn = (1:burnin)

## Acceptance rate
## apply(accept.coef[,-id.burn],1,mean)

## Effective sample size
hh = function(x) effectiveSize(x)
## cat("\nEffective sample sizes:\n")
## print(apply(theta.chn[,-id.burn],1,hh))

## Point estimation based on MCMC
theta.MAP = apply(theta.chn[,-id.burn],1,mean) ## All parameters, penalty parameters included
coef.MAP = theta.MAP[coef.lab] ## ## All parameters, except penalty parameters
regr.MAP = c(psiToRegr %*% theta.MAP[c(psi.lab,dbeta.lab)]) ## (beta,gamma) parameters
names(regr.MAP) = rownames(psiToRegr) ## ... with labels for output

## Credible intervals (CI) for different quantities
hpd.fun = function(x) HPDinterval(mcmc(x))[,c("lower","upper")]
coef.HPD = hpd.fun(t(theta.chn[c(psi.lab,dbeta.lab),-(1:burnin)])) ## CI for all parameters, except penalty parameters
regr.HPD = hpd.fun(t(regr.chn[,-(1:burnin)])) ## CI for regression parameters (beta,gamma)
if (exists("quant0")){
    beta0b.MAP = apply(beta0b.chn[,-id.burn],1,mean)
    beta0b.HPD = hpd.fun(t(beta0b.chn[,-(1:burnin)]))
}
## Monitoring of the correlation between regression parameters
cor.regr = regr.cor = round(cor(t(regr.chn[,-(1:burnin)])),3)
cor.psidpsi = round(cor(t(theta.chn[c(psi.lab,dbeta.lab),-id.burn])),3)
##

## print(summary(mcmc(t(regr.chn[,-id.burn]))))


