###############################################################################
## Please cite and acknowledge                                               ##
##  Lambert, P. and Bremhorst, V. (2018)                                     ##
##    Estimation and identification issues in the promotion time cure        ##
##    model when the same covariates influence long- and short-term survival ##
##  Biometrical Journal (2018)                                               ##
##                                                                           ##
## when using part of this code in your work and publications                ##
###############################################################################

####################################################################################################################
##                  Expected defined objects before sourcing this file:
####################################################################################################################
##
## * Data
## ** <obsTime>
##         vector of length <n> containing the event- or right-censored times
## ** <delta>
##         vector of length <n> containing the 0-1 event indicators
## ** <Design>
##         n by (p+1) design matrix (first column of 1 ; other columns with covariates involved
##                                   in short- or long-term survival model parts)
##         ADVICE: standardize continuous covariates (to have mean 0 and variance 1)
##
## ** <X.col>
##         (p+1)-vector with 0-1 indicating the columns of <Design> entering the long-term survival
##         model part (cf. Eq.(1) in the paper)
## ** <W.col>
##         (p+1)-vector with 0-1 indicating the columns of <Design> entering the short-term survival
##         model part (cf. Eq.(2) in the paper)
##
##         IMPORTANT NOTE: *** make sure that min(X.col+W.col)>=1 !!
##                         *** W.col[1] should be 0 (as no intercept in the short-term survival model part)
##
## ** <tmin> and <tmax>
##         Support of the event-time distribution: (tmin,tmax)
## ** <xl> and <xr>
##         Support of the censoring time distribution: (xl,xr)
##
## * Priors:
## ** <pcured.min> and <pcured.max>
##         beta0 = log(-log(P(cured|x=0) )) assumed to lie with prob 0.95 in (pcured.min,pcured.max)
## ** <s0.regr>
##         Std deviation of the regression parameters
##
## * Note: By default:
## ** <KK> = 10 knots taken on (tmin,tmax) for the baseline hazard function (see below)
## ** <pen.order> = 3 is the penalty order with P-splines (see below)
## ** <niter> = 1.2e4 is the number of iterations in MCMC (see file "FlexibleCureModel_BayesianEstimation.R")
## ** <burnin> = 2e3  is the length of burn-in (see file "FlexibleCureModel_BayesianEstimation.R")
## ** <targetRate> = .35 is the target acceptance rate in MCMC (see file "FlexibleCureModel_BayesianEstimation.R")
##
####################################################################################################################


####################################################################################################################
##               Expected output after sourcing "FlexibleCureModel_main.R":
####################################################################################################################
## * Printed table with summary of MCMC for the regression parameters <psi>, <dpsi>, <beta>, <gamma>
##  (Mean, SD, quantiles 50% 2.5% 5% 95% 97.5%, Pval=min(P(coef>0|data),P(coef<0|data)), effective MCMC chain length)
##
## * <theta.chn>:  <ntheta> by <niter> matrix with the MCMC chains for
## ** the modified spline parameters <zeta> where phi_k = phi.max - exp(zeta_k)
## ** the regression parameters <psi> and <dbeta> (i.e. dpsi in the paper),
## ** the penalty parameters <penalty1> (i.e. lambda) and <penalty2> (i.e. delta)
##
## * <regr.chn>: <nbeta+ngamma> by <niter> matrix with the MCMC chains for
## ** the regression parameters <beta> in the long-term survival model part
## ** the regression parameters <gamma> in the short-term survival model part
##
## * <coef.HPD>: 95% HPD credible intervals for <psi> and <dbeta> (i.e. dpsi)
## * <regr.HPD>: 95% HPD credible intervals for <beta> and <gamma>
##
## * <penalty.chn>: 2 by <niter> matrix with the MCMC chains for lambda and delta
##
####################################################################################################################


## Make sure that a prior for P(cured|x=0) was defined
if (!exists("pcured.min")){
    pcured.min = log(-log(.60))
    cat("Default prior assumption on maximum plausible cure probability has been made\n")
    cat("Set <p.cured.min = log(-log(Pcured.max))> to change this !\n")
}
if (!exists("pcured.max")) {
    pcured.max = log(-log(.01))
    cat("Default prior assumption on minimum plausible cure probability has been made\n")
    cat("Set <p.cured.max = log(-log(Pcured.min))> to change this !\n")
}
## beta[0] = log(-log(P(cured|x=0)) with prior on <beta0>:  N(b0.mean,b0.sd^2)
b0.mean = .5*(pcured.min + pcured.max) ; b0.sd = (pcured.max - pcured.min)/4

## Default std deviation for priors of regression coefficients
if (!exists("s0.regr")){
    s0.regr = 1.5
    cat("Default choice for prior Std dev. of regression parameters has been made\n")
    cat("Set <s0.regr> to change this !\n")
}

## B-spline function
require(splines)
cubicBsplines.general = function(x,knots){ ## Equidistant knots are assumed !!
    dknots = diff(knots)[1]
    knots2 = c(knots[1]-(3:1)*dknots,knots,tail(knots,1)+(1:3)*dknots)
    ans = splineDesign(knots2,x)
    return(ans)
}

## Sample size
n = length(obsTime)

### Flexible log-hazard on (tmin,tmax)
KK = 10  ## Default number of spline parameters
knots = seq(tmin,tmax,length=KK-2) ## Equidistant knots
##
## Penalty matrix
pen.order = 3 ## Default penalty order
Dd = diff(diag(KK),dif=pen.order) ## Difference matrix
Pd = t(Dd)%*%Dd + 1e-6*diag(KK) ## Penalty matrix
sP = svd(Pd) ## For use in approx to p(tau|data)

## B-spline basis on a fine regular grid
n.grid = 200 ## Grid size
t.grid = seq(tmin,tmax,length=n.grid) ## Grid of time points
dt.grid = t.grid[2] - t.grid[1] ## Distance between grid points
B.grid = cubicBsplines.general(t.grid, knots) # B-spline matrix on grid

## Baseline hazard and cumulative hazard functions for given spline parameters <phi>
h0H0.fun = function(t,phi){
    ## B.t = cubicBsplines.general(t, knots) # B-spline matrix on grid
    ## lh0.t = c(B.t %*% phi) ## Log baseline hazard
    ## h0.t = exp(lh0.t) ## Baseline hazard on grid
    ##
    lh0.grid = c(B.grid %*% phi) ## Log baseline hazard on grid
    h0.grid = exp(lh0.grid) ## Baseline hazard on grid
    H0.grid = dt.grid*(cumsum(h0.grid)-.5*(h0.grid[1]+h0.grid)) ## Baseline cumulative hazard on grid
    ##
    h0.t = exp(approx(t.grid,lh0.grid,xout=t)$y) ## Baseline hazard at t
    H0.t = approx(t.grid,H0.grid,xout=t)$y ## Baseline cumulative hazard at t
    ## h0.t = pmax(pmin(exp(splinefun(t.grid,lh0.grid)(t)),1e6),1e-6) ## Baseline hazard at t
    ## H0.t = pmax(pmin(splinefun(t.grid,H0.grid)(t),1e6),1e-6) ## Baseline cumulative hazard at t
    return(list(h0=h0.t,H0=H0.t))
}

## Reparametrization of splines parameters <phi> for better mixing during MCMC
##    phi_k = phi.max-exp(zeta_k)
zeta2phi = function(zeta,phi.max=2) phi.max-exp(zeta)
phi2zeta = function(phi,phi.max=2) log(-phi+phi.max)

## Log-posterior ignoring covariates and censored times
## Input:
##        alpha: vector of spline, regression and penalty parameters
##        obsTime: vector of event or censoring times
##        delta: vector of event indicators
## Ouput:
##        log-posterior based on event times only (delta=1) and ignoring covariates
##
logpost0 = function(alpha,obsTime,delta,phi.max=2){
    zeta = alpha[1:KK] ; phi = zeta2phi(zeta,phi.max=phi.max) ## Spline parameters
    if (any(!is.finite(phi))) return(-1e6)
    tau = abs(alpha[KK+1]) ## Penalty parameter
    ##
    idx = which(delta==1) # Restrict to events
    temp = h0H0.fun(obsTime[idx],phi)
    lh0.t = log(temp$h0) ; H0.t = temp$H0
    lhp.t = lh0.t - H0.t
    lSp.t = -H0.t
    ##
    llik = sum(delta[idx]*lhp.t + lSp.t)
    ## Computation of  log p(phi|tau) + log(det(Sigma.tau))
    ## Use log(det(Sigma.tau)) \upto -.5*K*log(tau)-sum_k log(1+n*dk/tau)
    ##   (cf. approx marginal post of tau p(tau|data) )
    lpen = - .5*tau*sum(zeta*c(Pd%*%zeta)) -.5*sum(log(1+n*d.k/tau))
    ##
    ## Encourage S0(tmax) in (1e-6, 1e-2)
    Stmax = exp(-h0H0.fun(tmax,phi)$H0) ## Value of S0(tmax)
    Kpen = 5 ## Penalty parameter used to force S0(tmax)=Stmax approx 0
    lpen2 = Kpen*dnorm(log10(Stmax),-4,1,log=T)
    ##
    ## Prior on penalty parameter <tau> (Jullion-Lambert, CSDA 2007)
    lpen3 = -.5*log(tau)-log(1+tau) ## Jullion-Lambert with nu=1, a.delta=b.delta=.5
    ##
    lpost = llik + lpen + lpen2 + lpen3
    names(lpost) = "lpost0"
    return(lpost)
}

## Joint log-posterior  logLik + log p(phi|tau) + log p(tau): general form
##
## Input:
##        alpha: vector of spline, regression and penalty parameters
##        obsTime: vector of event or censoring times
##        delta: vector of event indicators
##
## Implicit (i.e. defined in the calling environment):
##        an underlying design matrix <Design> with <X.col> & <W.col>
##        to indicate what columns in <Design> are actived
##        in long- and short-term survival model parts, respectively
##
## Ouput:
##        log-posterior for the flexible promotion time cure model
##
logpost = function(alpha,obsTime,delta,phi.max=2,noprior=FALSE){
    ## Spline parameters
    zeta = alpha[1:KK] ; phi = phi = zeta2phi(zeta,phi.max=phi.max)
    if (any(!is.finite(phi))) return(-1e10)
    psi = alpha[(KK+1):(KK+ncol(Design))] ## Regression parameters <psi>
    ## dbeta (denoted by <dpsi> in the paper)
    dbeta = 0*psi
    ndbeta = sum(X.col*W.col) ## Number of shared covariates
    if (ndbeta > 0) dbeta[(X.col*W.col)==1] = alpha[(KK+ncol(Design)+1):(KK+ncol(Design)+ndbeta)]
    ## Penalty parameter
    tau = abs(alpha[KK+ncol(Design)+ndbeta+1])
    ## Regression parameters
    beta = psi*X.col + dbeta  ## Regression parameters <beta> (long-term survival)
    gamma = psi*W.col - dbeta ## Regression parameters <gamma> (short-term survival)
    ## Linear predictors
    eta1 = c(Design%*%beta) ## log(theta(x)) = linear predictor in long-term survival model (cf. Eq.(1) in paper)
    eta2 = c(Design%*%gamma)## z'gamma = linear predictor in short-term survival model (cf. Eq.(2) in paper)
    temp = h0H0.fun(obsTime,phi)
    lh0.t = log(temp$h0) ; H0.t = temp$H0 ## Baseline (log-)hazard and cumulative hazard
    lhp.t = eta1 + eta2 + lh0.t - exp(eta2)*H0.t ## Population (log-)hazard
    lSp.t = -exp(eta1)*(1-exp(-exp(eta2)*H0.t))  ## Population (log-)survival
    ## Log-likelihood computation
    idx0 = which(delta==0) ; idx1 = which(delta==1)
    llik = 0
    llik = llik + sum(lSp.t[idx0]) ## (log-)likelihood contribution when right-censored
    llik = llik + sum(lhp.t[idx1] + lSp.t[idx1]) ## (log-)likelihood contribution when observed event
    ##
    LogSumF0 = log(sum(1-exp(-H0.t)))
    ## Computation of  log p(phi|tau) + log(det(Sigma.tau))
    ## Use log(det(Sigma.tau)) \upto -.5*K*log(tau)-sum_k log(1+n*dk/tau)
    ##   (cf. approx marginal post of tau p(tau|data) )
    lpen = -.5*tau*sum(zeta*c(Pd%*%zeta)) -.5*sum(log(1+n*d.k/tau))
    ##
    ## Encourage S0(tmax) in (1e-5, 1e-1)
    Stmax = exp(-h0H0.fun(tmax,phi)$H0)
    Kpen = 5 ## Penalty parameter used to force S0(tmax)=Stmax approx 0
    lpen2 = Kpen*dnorm(log10(Stmax),-3,1,log=T)
    ## Prior on penalty parameter <tau> (Jullion-Lambert, CSDA 2007)
    lpen3 = -.5*log(tau)-log(1+tau) ## Jullion-Lambert with nu=1, a.delta=b.delta=.5
    ##
    lprior = 0
    ## Prior on <psi>
    lprior = lprior + dnorm(psi[1],b0.mean,b0.sd,log=T)   ## psi[1] = beta0 = log(-log(P(cured|x=0)))
    lprior = lprior + sum(dnorm(psi[-1],0,s0.regr,log=T)) ## Prior on <psi[-1]>
    ## Prior on <dbeta> (cf. <dpsi> in the paper)
    if (ndbeta > 0){
        parms = dbeta[(X.col*W.col)==1]
        lpen.dbeta = sum(dnorm(parms,0*parms,s0.regr,log=T))
        lprior = lprior + lpen.dbeta
    }
    ## (Log-)posterior computation
    if (noprior) lprior = 0
    lpost = llik + lpen + lpen2 + lpen3 + lprior ## (log-)posterior
    names(lpost) = "lpost"
    attr(lpost,"llik") = llik
    attr(lpost,"LogSumF0") = LogSumF0
    ##
    if (!is.finite(lpost)) return(-1e12)
    return(lpost)
}

## Function to report aggregated simulation results
extractInfo.fun = function(obj){
    alpha.hat = obj$est ; names(alpha.hat) = alpha.lab
    nalpha = length(alpha.hat)
    psi.hat = alpha.hat[psi.lab]
    id.coef = (KK+1):(nalpha-1)
    coef.hat = alpha.hat[id.coef]
    Sigma.obj = solve(obj$hes[id.coef,id.coef])
    coef.se = sqrt(diag(Sigma.obj)) ; names(coef.se) = alpha.lab[id.coef]
    coef.z = coef.hat / coef.se
    coef.025 = coef.hat - 1.96*coef.se
    coef.975 = coef.hat + 1.96*coef.se
    coef.error = coef.hat-coef.true
    mat.coef = round(cbind(coef.true,coef.hat,coef.error,coef.se,coef.z,coef.025,coef.975),3)
    ##
    coef.in = (coef.true>coef.025)*(coef.true<coef.975)
    mat.coef = cbind(mat.coef,coef.in)
    ##
    zeta.hat = alpha.hat[zeta.lab]
    phi.hat = phi = zeta2phi(zeta.hat,phi.max=phi.max) ##phi.max - exp(zeta.hat) ;
    ## (psi,dbeta).hat to (beta,gamma).hat
    regr.hat = c(psiToRegr %*% coef.hat)
    Sigma.regr = psiToRegr %*% Sigma.obj %*% t(psiToRegr)
    regr.se = sqrt(diag(Sigma.regr))
    regr.z = regr.hat / regr.se
    names(regr.hat) = names(regr.se) = regr.lab
    regr.025 = regr.hat - 1.96*regr.se
    regr.975 = regr.hat + 1.96*regr.se
    regr.error = regr.hat-regr.true
    regr.in = (regr.true>regr.025)*(regr.true<regr.975)
    ##
    mat.regr = round(cbind(regr.true,regr.hat,regr.error,regr.se,regr.z,regr.025,regr.975),3)
    mat.regr = cbind(mat.regr,regr.in=regr.in)
    ##
    S0quant0.hat = exp(-h0H0.fun(quant0,phi.hat)$H0)
    beta0b.hat = regr.hat["beta0"] + log(1-S0quant0.hat) ; names(beta0b.hat) = "beta0b"
    ##
    tau.hat = abs(tail(obj$est,n=1)) ##  <-- abs(.) !!
    ##
    return(list(mat.coef=mat.coef,mat.regr=mat.regr,beta0b.hat=beta0b.hat,S0quant0.hat=S0quant0.hat,tau.hat=tau.hat))
}

## Function to summarize estimation results
extractInfo2.fun = function(obj){
    alpha.hat = obj$est ; names(alpha.hat) = alpha.lab
    nalpha = length(alpha.hat)
    psi.hat = alpha.hat[psi.lab]
    id.coef = (KK+1):(nalpha-1)
    coef.hat = alpha.hat[id.coef]
    Sigma.obj = solve(obj$hes[id.coef,id.coef])
    coef.se = sqrt(diag(Sigma.obj)) ; names(coef.se) = alpha.lab[id.coef]
    coef.z = coef.hat / coef.se
    coef.025 = coef.hat - 1.96*coef.se
    coef.975 = coef.hat + 1.96*coef.se
    coef.pval = 1-pchisq(coef.z^2,1)
    mat.coef = round(cbind(coef.hat,coef.se,coef.z,coef.025,coef.975,coef.pval),3)
    ##
    zeta.hat = alpha.hat[zeta.lab]
    phi.hat = phi = zeta2phi(zeta.hat,phi.max=phi.max) ##phi.max - exp(zeta.hat) ;
    names(phi.hat) = phi.lab
    ## (psi,dbeta).hat to (beta,gamma).hat
    regr.hat = c(psiToRegr %*% coef.hat)
    Sigma.regr = psiToRegr %*% Sigma.obj %*% t(psiToRegr)
    regr.se = sqrt(diag(Sigma.regr))
    regr.z = regr.hat / regr.se
    regr.pval = 1-pchisq(regr.z^2,1)
    names(regr.hat) = names(regr.se) = regr.lab
    regr.025 = regr.hat - 1.96*regr.se
    regr.975 = regr.hat + 1.96*regr.se
    ##
    mat.regr = round(cbind(regr.hat,regr.se,regr.z,regr.025,regr.975,regr.pval),3)
    ##
    tau.hat = abs(tail(obj$est,n=1)) ##  <-- abs(.) !!
    ##
    return(list(mat.coef=mat.coef,mat.regr=mat.regr,tau.hat=tau.hat))
    ##return(list(mat.coef=mat.coef,mat.regr=mat.regr,beta0b.hat=beta0b.hat,S0quant0.hat=S0quant0.hat,tau.hat=tau.hat))
}

########################
## Parameter Estimation
########################
##
## Bs: B-spline basis at event times obsTime[delta==1]
## dk: Eigenvalues to compute the approx to log(deta(Sigma.tau)) in p(tau|data)
Bs = cubicBsplines.general(obsTime[delta==1], knots)
AA = Bs %*% sP$u %*% diag(sP$d/sqrt(n))
d.k = svd(t(AA)%*%AA)$d ## log(det(Sigma.tau)) \upto -.5*K*log(tau)-sum_k log(1+n*dk/tau)

## Parameter labels and reparametrization
## --------------------------------------
## Labels for spline parms
phi.lab = paste("phi",1:KK,sep="")
zeta.lab = paste("zeta",1:KK,sep="")
##
## Labels for regression parms
psi.lab = paste("psi",0:(ncol(Design)-1),sep="")
dbeta0 = dbeta.lab = NULL
ndbeta = sum(X.col*W.col)
dbeta.id = (0:(ncol(Design)-1))[X.col*W.col==1]
if (ndbeta > 0){
    dbeta.lab = paste("dbeta",dbeta.id,sep="")
    dbeta0 = rep(0,ndbeta)
}
##
ndesign = ncol(Design)    # Nbr of columns in the Design matrix
nbeta = sum(X.col!=0)     # Nbr of <beta> coefs to estimate
ngamma = sum(W.col!=0)    # Nbr of <gamma> coefs to estimate
ndbeta = sum(X.col*W.col) # Nbr of covariates shared by long- and short-term regression models
nest = ndesign + ndbeta   # Nbr of estimated regression parameters
##
beta.lab = paste("beta",(0:(ncol(Design)-1))[X.col==1],sep="")
gamma.lab = NULL
if (sum(W.col==1)>0) gamma.lab = paste("gamma",(0:(ncol(Design)-1))[W.col==1],sep="")
regr.lab = c(beta.lab,gamma.lab)
penalty.lab = "penalty"
##
alpha.lab = c(zeta.lab,psi.lab,dbeta.lab,penalty.lab)

## Matrix yielding regr coef (beta,gamma) from (psi,dbeta)
##   c(beta,gamma) = psiToRegr %*% c(psi,dbeta)
## Psi to Regr
psiToRegr = matrix(0, nrow=nbeta+ngamma, ncol=nest)
rownames(psiToRegr) = c(beta.lab,gamma.lab)
colnames(psiToRegr) = c(psi.lab,dbeta.lab)
psiToRegr[cbind(1:nbeta,(1:ndesign)[X.col==1])] = 1
psiToRegr[cbind((nbeta+1):(nbeta+ngamma),(1:ndesign)[W.col==1])] = 1
if (ndbeta > 0){
    psiToRegr[cbind(paste("beta",dbeta.id,sep=""),paste("dbeta",dbeta.id,sep=""))]  = 1
    psiToRegr[cbind(paste("gamma",dbeta.id,sep=""),paste("dbeta",dbeta.id,sep=""))]  = -1
}

## Initial values (ignoring covariates and censored observations)
## --------------
## Initial values for the spline parameters
phi00 = beta00 = (rep(1/mean(obsTime),KK)) ## ... in the <phi> parametrization
phi.max = 2 ; zeta00 = phi2zeta(phi00,phi.max=phi.max) ## ... in the <zeta> parametrization
##
## Clever starting value for beta0[1] = psi0[1] = log(-log(P(cured|x=0))) ^= log(-log(K-M estimate at right tail))
km0 = survfit(Surv(obsTime,delta)~1)
psi0 = rep(0,ncol(Design))
psi0[1] = log(-log(max(tail(km0$surv,n=1),1/n))) ## log(-log(max(tail(km0$surv,n=1),1/n)))
##
## Reasonable initial values for the spline parameters in log(h_0(t))
g0 = function(x) return(-logpost0(x,obsTime,delta))
obj0 = nlm(g0,c(zeta00,10),hessian=F,iterlim=200)  ## Maximize the posterior based on event times (with x=0)
zeta0 = obj0$est[1:KK] ## Estimated starting values for the spline parameters
penalty0 = obj0$est[KK+1] ## Estimated starting value for the penalty parameter
##
## Starting values for the model parameters, in summary:
if (exists("regr.true")){ ## Case 1: simulation study
    alpha0 = c(zeta0,c(solve(psiToRegr)%*%regr.true),penalty0)
} else {                  ## Case 2: case study
    alpha0 = c(zeta0,rep(0,ncol(psiToRegr)),penalty0)
}
names(alpha0) = alpha.lab ## ... with labels
nalpha = length(alpha0)

## Rough FREQUENTIST estimation
## ----------------------------
cat("\n== Rough frequentist estimation ==\n")
##
g1 = function(alpha){
    tau = abs(tail(alpha,n=1))
    ans = - logpost(alpha,obsTime,delta)
    return(ans)
}
## Rough point estimation of the model parameters
##  (for further use as starting values in MCMC)
obj1 = try(nlm(g1,alpha0,typsize=.1*alpha0,hessian=T,iterlim=500))
if (exists("regr.true")){ ## Case 1: simulation study
    res1 = extractInfo.fun(obj1)
} else {                  ## Case 2: case study
    res1 = extractInfo2.fun(obj1)
}
cat("tau.hat=",abs(round(tail(obj1$est,n=1),1)),"\n")
print(res1$mat.regr)

## BAYESIAN estimation
## -------------------
cat("\n== Bayesian estimation ==\n")

## Metropolis-within-Gibbs algorithm
source("FlexibleCureModel_BayesianEstimation.R")

## Function to compute post prob that Coef >0 or <0
pval.fun = function(x){
    prob.ge.0 = sum(x>0)/length(x)
    return(round(min(prob.ge.0,1-prob.ge.0),3))
}
require("gtools")

## Generic function to summarize information from MCMC chain
EstimationSummary.fun = function(coef.chn,p=c(.025,.05,.5,.95,.975)){
    res = t(apply(coef.chn,1,quantile,p=p))
    pvals = apply(coef.chn,1,pval.fun)
    ## Generates '*', '**', '***' to points small pvals
    temp = stars.pval(pvals)
    ## Returns everything as a data.frame
    require("dplyr") ## Required to rename dataframe components
    ##
    ## Also report 2.5%, 5%, 95%, 97.5% quantiles
    pvals.df = data.frame(Var="",
                          Coef=names(pvals),
                          Mean=round(apply(coef.chn,1,mean),3),
                          SD=round(apply(coef.chn,1,sd),3),
                          round(unname(res[,c(3,1,2,4,5)]),3),
                          Pval=unname(pvals),
                          Sig=unname(stars.pval(pvals)),
                          EffSize=round(apply(coef.chn[,-id.burn],1,effectiveSize))
                          )
    pvals.df = rename(pvals.df,"50%"=X1,"2.5%"=X2,"5%"=X3,"95%"=X4,"97.5%"=X5)
    attr(pvals.df,"legend") = attr(temp,"legend")
    rownames(pvals.df) = names(pvals)
    return(pvals.df)
}

## Estimation summary for regression coefs psi-dpsi
options(width=120)  ## Increase width of printing area
summaryBayesEst.psidpsi = EstimationSummary.fun(theta.chn[c(psi.lab,dbeta.lab),-id.burn])
summaryBayesEst.psidpsi$Var = colnames(Design)[c(which(X.col+W.col>=1),which(X.col*W.col==1))]
print(summaryBayesEst.psidpsi)

## Estimation summary for regression coefs beta-gamma
summaryBayesEst.regr = EstimationSummary.fun(regr.chn[,-id.burn])
summaryBayesEst.regr$Var = colnames(Design)[c(which(X.col==1),which(W.col==1))]
print(summaryBayesEst.regr)
