###############################################################################
## Please cite and acknowledge                                               ##
##  Lambert, P. and Bremhorst, V. (2018)                                     ##
##    Estimation and identification issues in the promotion time cure        ##
##    model when the same covariates influence long- and short-term survival ##
##  Biometrical Journal (2018)                                               ##
##                                                                           ##
## when using part of this code in your work and publications                ##
###############################################################################

require(survival)
options(warn = -1)

#############################################################
## ---> MODIFY HERE to choose a specific simulation setting
#############################################################
## Correctly specified or Mispecified model ?
CorrectlySpecifiedModel = FALSE  ##  Make your choice here !
                                 ##  TRUE: Table 1 in the paper
                                 ##  FALSE: Tables 2 or 3 in the paper

## Number of datasets / Sample size
Nsimul = 500  ## Number of replicates (500 in the paper)
n = 500 ## Sample size (300 or 500 in the paper)

## Maximum follow-up time (denoted by t_R in the paper)
xr = 25 # 10 15 25  ## (cf. simulation Setting in Tables 1,2,3)

#############################################################
## DO NOT CHANGE CODE BELOW this line !!!!
#############################################################

## Priors in the simulation study
###################################

## Prior on P(cured|x=0) (such that in (p0.min,p0.max) with prob 0.95)
p0.min = .01 ; p0.max = .30 ## Chosen values during simulation

## P(cured|x=0): induced prior on <beta0>:  N(b0.mean,b0.sd^2)
pcured.min = log(-log(p0.max)) ; pcured.max = log(-log(p0.min))  ## Range for P(cured|x=0)
b0.mean = .5*(pcured.min + pcured.max) ; b0.sd = (pcured.max - pcured.min)/4

## Default std deviation for priors of regression coefficients during simulation
s0.regr = 1.5

######################
## Simulation study ##
######################

## Seeds for each of the <Nsimul> replicates
seeds = 124 + (1:Nsimul) - 1

## Support of the event time distribution
tmin = 0 ; tmax = 30

## (xl,xr): Interval during which right censoring process works
xl = 0
## xr = 25 #10 15 25  ## To be modified

## Parameters of the Weibull distribution for the censoring times
a_Censure = 3
b_Censure = 25

## Parameters of the latent (Weibull) distribution for the event times
eta = exp(-4.4) ; rho = 2
a = rho ; b = 1/eta^(1/rho)
F0.true = function(x) pweibull(x,a,b) ## Baseline (i.e. when x=0) cdf of carcinogenic cell
Q0.true = function(q) qweibull(q,a,b) ## Baseline (i.e. when x=0) quantile of carcinogenic cell
temp = c(.05,seq(.1,.9,by=.1),.95,.9995)
quant0 = Q0.true(temp) ; names(quant0) = temp ## Quantiles to estimate

##### Loop over datasets/ replicates
for (s in 1:Nsimul){
    ##
    cat("\n -- Simulation number ", s, " --\n")
    pcured.vec = prcens.vec = NULL
    ##
    ## -1- Data generation: following Bremhorst & Lambert (CSDA 2016)
    ## ####################
    ##
    if (CorrectlySpecifiedModel) source("datasetGeneration_CorrectlySpecifiedModel.R")
    else source("datasetGeneration_MisspecifiedModel.R")
    ## beta0b = psi0 + log(F0(quant0))  with quant0 = Q0.true(c(.05,seq(.1,.9,by=.1),.95))
    beta0b.true = regr.true["beta0"] + log(F0.true(quant0))
    names(beta0b.true) = paste("beta0b","-",names(quant0),sep="")
    ##
    ## -2- Parameter Estimation
    ###########################
    ##
    source("FlexibleCureModel_main.R")
    ##
    ## Space to save posterior correlation
    if (s==1) regr.cor.chn = array(dim=c(length(regr.lab),length(regr.lab),Nsimul))

    ## Save approximate frequentist estimation results
    ## -----------------------------------------------
    if (s==1){
        ## Space for saving estimation results
        ## (1) Frequentist results
        alpha.mat1 = coef.mat1 = regr.mat1 = NULL # Matrix for storage of parameter estimates
        ##
        coef.in.mat1 = regr.in.mat1 = NULL # Matrix for storage of indicators that CI contains true value
        ##
        ##
        beta0b.mat = NULL  ## Bias corrected beta0 values
        S0quant0.mat = NULL
        logSumF0.vec = NULL
        tau.vec = NULL
        ##
        ## (2) Bayesian results
        theta.MAP.mat = coef.MAP.mat = regr.MAP.mat = NULL
        coef.HPD.lower = coef.HPD.upper = coef.in.HPD = NULL
        regr.HPD.lower = regr.HPD.upper = regr.in.HPD = NULL
        beta0b.HPD.lower = beta0b.HPD.upper = beta0b.in.HPD = NULL
        beta0b.MAP.mat = NULL
    }
    ## Save approximate frequentist estimation results
    ## -----------------------------------------------
    coef.mat1 = cbind(coef.mat1,res1$mat.coef[,"coef.hat"])
    beta0b.mat = c(beta0b.mat,res1$beta0b.hat)
    S0quant0.mat = cbind(S0quant0.mat,res1$S0quant0.hat)
    logSumF0.vec = c(logSumF0.vec,attr(g1(obj1$est),"LogSumF0"))
    tau.vec = c(tau.vec,tail(obj1$est,n=1))
    ## Model parameters
    alpha.mat1 = cbind(alpha.mat1,obj1$est)
    ## Regression parameters (beta_k,gamma_k)
    regr.mat1 = cbind(regr.mat1,res1$mat.regr[,"regr.hat"])
    ## Indicators that CI contains true value
    coef.in.mat1 = cbind(coef.in.mat1,res1$mat.coef[,"coef.in"])
    regr.in.mat1 = cbind(regr.in.mat1,res1$mat.regr[,"regr.in"])
    ##
    ## Save Bayesian estimation results
    ## --------------------------------
    theta.MAP.mat = cbind(theta.MAP.mat,theta.MAP)
    coef.MAP.mat = cbind(coef.MAP.mat,coef.MAP)
    regr.MAP.mat = cbind(regr.MAP.mat,regr.MAP)
    ##
    beta0b.MAP.mat = cbind(beta0b.MAP.mat,beta0b.MAP)
    beta0b.HPD.lower= cbind(beta0b.HPD.lower,beta0b.HPD[,"lower"])
    beta0b.HPD.upper= cbind(beta0b.HPD.upper,beta0b.HPD[,"upper"])
    beta0b.true = regr.true["beta0"] + log(F0.true(quant0))
    temp = apply(beta0b.HPD-beta0b.true,1,function(x) sign(prod(x))<0)
    beta0b.in.HPD = cbind(beta0b.in.HPD,c(0+temp))
    ##
    regr.HPD.lower= cbind(regr.HPD.lower,regr.HPD[,"lower"])
    regr.HPD.upper= cbind(regr.HPD.upper,regr.HPD[,"upper"])
    temp = apply(regr.HPD-regr.true,1,function(x) sign(prod(x))<0)
    regr.in.HPD = cbind(regr.in.HPD,0+temp)
    regr.cor.chn[,,s] = regr.cor
    ##
    coef.HPD.lower= cbind(coef.HPD.lower,coef.HPD[,"lower"])
    coef.HPD.upper= cbind(coef.HPD.upper,coef.HPD[,"upper"])
    temp = apply(coef.HPD-coef.true,1,function(x) sign(prod(x))<0)
    coef.in.HPD = cbind(coef.in.HPD,0+temp)
    ##
    ## Hist of marginal posteriors
    plot.marginalPost = FALSE
    if (plot.marginalPost){
        par(mfrow=c(ceiling(length(regr.true)/2),2),mar=c(4,4,1,1))
        par(oma=c(0,0,2,0))
        for (k in 1:length(regr.true)){
            hist(regr.chn[k,-(1:burnin)],freq=F,xlab=names(regr.true)[k],main="",fill=T,col="grey",breaks=25)
            points(c(regr.HPD[k,],regr.MAP[k]),c(0,0,0),pch="X",cex=1.1)
            abline(v=regr.true[k],col="red",lwd=2)
        }
        title(main=paste("Simulation dataset No ",s,sep=""),outer=T)
    }
    ## Provide summary of simulation results up to now
    ## -----------------------------------------------
    if (s > 1){ ## Start that when at least 2 datasets analyzed...
        rownames(alpha.mat1) = alpha.lab
        rownames(regr.mat1) = regr.lab
        ## Generic function to communication simulation results
        summary.simulations.fun = function(coef.mat,coef.in.mat,coef.true,main=""){
            gg = function(x) t.test(x)$p.val
            ## Summarize simulation results on coefs
            coef.mean = apply(coef.mat,1,mean)
            coef.bias = coef.mean - coef.true
            ##
            coef.ese = sqrt(apply((coef.mat-coef.mean)^2,1,mean))
            coef.rmse = sqrt(apply((coef.mat-coef.true)^2,1,mean))
            coef.coverage = apply(coef.in.mat,1,mean,na.rm=T)
            ##
            coef.results = cbind(true=coef.true,mean=coef.mean,bias=coef.bias,ese=coef.ese,rmse=coef.rmse,cov.95=coef.coverage)
            if (main!=""){
                cat(main,"\n")
                print(round(coef.results,3))
            }
            return(coef.results)
        }
        ##
        cat("\n================================\n")
        cat("  Aggregated simulation results  \n")
        cat("      over",s,"datasets\n")
        cat("================================\n")
        cat("Sample size:",n,"\n")
        cat("Max follow-up time (xr):",xr,"\n")
        cat("Support assumed to be on (0,",tmax,")\n",sep="")
        cat("Average proportion of cured:",round(mean(pcured.vec),3),"\n")
        cat("Average proportion of right censored:",round(mean(prcens.vec),3),"\n\n")
        ##
        ## cat("** Frequentist results **\n")
        ## Communicate rough frequentist simulation results
        coef.summary1 = summary.simulations.fun(coef.mat1,coef.in.mat1,coef.true,main="\n-- Rough frequentist estimation results")
        regr.summary1 = summary.simulations.fun(regr.mat1,regr.in.mat1,regr.true,main=" ")
        ##
        if (bayes.estimation){
            cat("\n** Bayesian results **\n")
            rownames(regr.MAP.mat) = regr.lab
            ## Communicate Bayesian simulation results
            coef.bayes.summary1 = summary.simulations.fun(coef.MAP.mat[names(coef.true),],coef.in.HPD,coef.true,main="\n-- Bayesian estimation results")
            regr.bayes.summary1 = summary.simulations.fun(regr.MAP.mat,regr.in.HPD,regr.true,main=" ")
            beta0b.bayes.summary1 = summary.simulations.fun(beta0b.MAP.mat,beta0b.in.HPD,beta0b.true,main=" ")
        }
        ## Prepare object to save simulation results
        results = list(coef.mat1=coef.mat1,
                       n=n,Nsimul=Nsimul,seed=seeds,
                       xr=xr, tmax=tmax,
                       quant0=quant0, F0.true=F0.true, Q0.true=Q0.true,
                       KK=KK, B.grid=B.grid, t.grid=t.grid,
                       logpost=logpost,
                       psiToRegr=psiToRegr,
                       beta.lab=beta.lab,gamma.lab=gamma.lab,alpha.lab=alpha.lab,
                       zeta.lab=zeta.lab,phi.lab=phi.lab,
                       psi.lab=psi.lab,dbeta.lab=dbeta.lab,
                       penalty.lab=penalty.lab,regr.lab=regr.lab,
                       beta.true=beta.true,gamma.true=gamma.true,
                       coef.true=coef.true,regr.true=regr.true,
                       coef.mat1=coef.mat1,
                       tau.vec=tau.vec,
                       alpha.mat1=alpha.mat1,
                       regr.mat1=regr.mat1,
                       coef.in.mat1=coef.in.mat1,
                       regr.in.mat1=regr.in.mat1)
        ##
        if (bayes.estimation){
            regr.cor.array = regr.cor.chn
            results$theta.MAP.mat=theta.MAP.mat
            results$coef.MAP.mat=coef.MAP.mat
            results$regr.MAP.mat=regr.MAP.mat
            results$beta0b.HPD.lower=beta0b.HPD.lower
            results$beta0b.HPD.upper=beta0b.HPD.upper
            results$beta0b.in.HPD=beta0b.in.HPD
            results$coef.HPD.lower=coef.HPD.lower
            results$coef.HPD.upper=coef.HPD.upper
            results$coef.in.HPD=coef.in.HPD
            results$regr.HPD.lower=regr.HPD.lower
            results$regr.HPD.upper=regr.HPD.upper
            results$regr.in.HPD=regr.in.HPD
            results$regr.cor.array=regr.cor.array
            results$coef.summary1=coef.summary1
            results$coef.lab=coef.lab
            results$regr.summary1=regr.summary1
            results$regr.bayes.summary1=regr.bayes.summary1
            ## Below: chains for the last database
            results$regr.chn=regr.chn
            results$theta.chn=theta.chn
            results$S0quant0.chn=S0quant0.chn
            results$beta0b.chn = beta0b.chn
            results$accept.rate=apply(accept.coef[,-id.burn],1,mean)
            results$burnin=burnin
            results$niter=niter
        }
    }
}

## Save results to file
if (CorrectlySpecifiedModel){
    file.sav = paste("Simulation_CorrectlySpecified_xr",xr,"n",n,".sav",sep="")
} else {
    file.sav = paste("Simulation_Misspecified_xr",xr,"n",n,".sav",sep="")
}
## save(results,file=file.sav)

