###############################################################################
## Please cite and acknowledge                                               ##
##  Lambert, P. and Bremhorst, V. (2018)                                     ##
##    Estimation and identification issues in the promotion time cure        ##
##    model when the same covariates influence long- and short-term survival ##
##  Biometrical Journal (2018)                                               ##
##                                                                           ##
## when using part of this code in your work and publications                ##
###############################################################################

## Simulation study (Section 3 in the paper)
## Dataset generation (Covariate selection misspecified)

## Regression model specification
design.gen = function(seed){
    set.seed(seed)
    ## Regressors
    X1 = rnorm(n, 0, 1)
    X2 = rnorm(n, 0, 1)
    X3 = rnorm(n, 0, 1)
    ## Design matrix
    Design = cbind(Intcpt = 1, X1=X1, X2=X2, X3=X3)
    ## Regression parameters
    beta.true  = c(.75,-.5,.8,1e-6) ; names(beta.true) = paste()
    gamma.true = c(0.,1e-6,.4,-.4)
    ## Labels for the regression parameters
    ndesign = ncol(Design)
    beta.lab = paste("beta",(0:(ndesign-1)),sep="")
    gamma.lab = paste("gamma",(0:(ndesign-1)),sep="")
    names(beta.true) = beta.lab
    names(gamma.true) = gamma.lab
    ##
    return(list(Design=Design,beta.true=beta.true,gamma.true=gamma.true))
}

## Design matrix generation
design.list = design.gen(seeds[s])
Design = design.list$Design
ndesign = ncol(Design)
##
beta.true  = design.list$beta.true
gamma.true = design.list$gamma.true
##
X.col = 0 + (beta.true!=0)
W.col = 0 + (gamma.true!=0)
##
psi.true = (beta.true + gamma.true) / (X.col + W.col)
names(psi.true) = paste("psi",(0:(ndesign-1)),sep="")
dbeta.true = (beta.true - psi.true) * (X.col * W.col)
names(dbeta.true) = paste("dbeta",(0:(ndesign-1)),sep="")
coef.true = c(psi.true,dbeta.true[(X.col * W.col) == 1])
regr.true = c(beta.true[X.col==1],gamma.true[W.col==1])
##
nbeta = sum(X.col!=0)     # Nbr of beta coefs to estimate
ngamma = sum(W.col!=0)    # Nbr of gamma coefs to estimate
ndbeta = sum(X.col*W.col) # Nbr of common covariates
nest = ndesign + ndbeta # Nbr of (regr) parameters truly estimated
## Matrix yielding regr coef (beta,gamma) from (psi,dbeta)
##   c(beta,gamma) = psiToRegr %*% c(psi,dbeta)
psiToRegr = matrix(0, nrow=nbeta+ngamma, ncol=nest)
beta.lab = paste("beta",(0:(ndesign-1))[X.col==1],sep="")
gamma.lab = paste("gamma",(0:(ndesign-1))[W.col==1],sep="")
psi.lab = paste("psi",(0:(ndesign-1)),sep="")
dbeta.id = (0:(ndesign-1))[X.col*W.col==1]
dbeta.lab = NULL
if (ndbeta > 0){
    dbeta.lab = paste("dbeta",dbeta.id,sep="")
}
rownames(psiToRegr) = c(beta.lab,gamma.lab)
colnames(psiToRegr) = c(psi.lab,dbeta.lab)
psiToRegr[cbind(1:nbeta,(1:ndesign)[X.col==1])] = 1
psiToRegr[cbind((nbeta+1):(nbeta+ngamma),(1:ndesign)[W.col==1])] = 1
if (ndbeta > 0){
    psiToRegr[cbind(paste("beta",dbeta.id,sep=""),paste("dbeta",dbeta.id,sep=""))]  = 1
    psiToRegr[cbind(paste("gamma",dbeta.id,sep=""),paste("dbeta",dbeta.id,sep=""))]  = -1
}
##
theta <- exp(Design%*%beta.true)
##
Cox_Cov <- exp(Design%*%gamma.true)
##
Nlatent <- c()
obsTime <- c()
delta <- rep(1, n)
##
for(i in 1:n) {
    Nlatent[i] <- rpois(1, theta[i])
    ##
    if(Nlatent[i] == 0) {
        obsTime[i] <- 99
    }
    else {
        u <- runif(Nlatent[i], 0, 1)
        t <- (-log(1-u))/Cox_Cov[i]
        t <- (t^(1/a))*b
        obsTime[i] <- min(t)
    }
}
##
## Censoring
## Parameters of the Weibull distribution for the censoring time
a_Censure <- 3
b_Censure <- 25
##
pCmax = pweibull(xr, a_Censure, b_Censure)
C = qweibull(runif(n)*pCmax, a_Censure, b_Censure)
##
censored <- which(obsTime > C)
obsTime[censored] <- C[censored]
delta[censored] <- 0
##
## Stat descr on the data
if (!exists("verbose")) verbose = TRUE
if (verbose){
    cat("--------------------------------------------\n")
    cat("Sample size:",n,"\n")
    pcured = round(sum(Nlatent==0)/n,2)
    cat("Max follow-up time (xr):",xr,"\n")
    cat("Right censoring: Weibull(3,25) truncated on (0,xr)\n")
    cat("Support assumed to be on (0,",tmax,")\n",sep="")
    cat("Proportion of cured:",pcured,"\n") ; pcured.vec = c(pcured.vec,pcured)
    prcens = round(1-sum(delta)/n,2)
    cat("Proportion of right censored:",prcens,"\n") ; prcens.vec = c(prcens.vec,prcens)
    cat("--------------------------------------------\n\n")
}
